SystemOrganization addCategory: #FuelPreview!

FLAnalysis subclass: #FLPreviewAnalysis
	instanceVariableNames: 'elementByObjectMap currentNode edges traceForPreview'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPreview'!

!FLPreviewAnalysis commentStamp: 'MartinDias 12/4/2011 11:04' prior: 0!
I am an analysis specialized for doing a visualization of the graph.

Example of use:

(FLAnalyzer newDefault showPreview 
	analysisFor: (SortedCollection sortBlock: [:a :b | a > b ])) 
	open!

!FLPreviewAnalysis methodsFor: 'private' stamp: 'MartinDias 12/23/2011 21:12'!
colorFor: anObject

	^ anObject == root
		ifTrue: [Color blue]
		ifFalse: [Color black]! !

!FLPreviewAnalysis methodsFor: 'analyzing' stamp: 'MartinDias 12/23/2011 20:30'!
edgePathTo: anObject 

	| edge |
	edge := edges detect: [:e | e to model = anObject].

	edge from model = root
		ifTrue: [^ {edge}]
		ifFalse: [^ (self edgePathTo: edge from model), {edge} ]! !

!FLPreviewAnalysis methodsFor: 'accessing' stamp: 'MartinDias 12/22/2011 01:40'!
edges
	^ edges! !

!FLPreviewAnalysis methodsFor: 'initialization' stamp: 'MartinDias 12/4/2011 01:52'!
initialize
	super initialize.
	elementByObjectMap := IdentityDictionary new.
	edges := OrderedCollection new.! !

!FLPreviewAnalysis methodsFor: 'analyzing' stamp: 'MartinDias 12/23/2011 20:59'!
mapAndTrace: anObject

	| aCluster |
	aCluster := self clusterFor: anObject.
	
	traceForPreview := false. "don't add to view the cluster-level references"
	clusterization add: aCluster traceWith: self.

	traceForPreview := true.
	currentNode := self nodeFor: anObject.
	aCluster add: anObject traceWith: self.
! !

!FLPreviewAnalysis methodsFor: 'private' stamp: 'MartinDias 12/4/2011 02:07'!
newEdgeTo: aReferencedObject
	^ (ROEdge from: currentNode to: (self nodeFor: aReferencedObject))
		addDecorator: (ROLine new color: Color green);
		yourself.! !

!FLPreviewAnalysis methodsFor: 'private' stamp: 'MartinDias 12/23/2011 21:20'!
newNodeFor: anObject
	^ (ROElement on: anObject)
			extent: 15@15;
			addDecorator: (ROCircle new color: (self colorFor: anObject));
			addInteraction: RODraggable;
			addInteraction: (ROPopup text: #printString);
			on: ROMouseRightClick do: [ :e | anObject inspect ];
			yourself! !

!FLPreviewAnalysis methodsFor: 'compiling' stamp: 'MartinDias 12/4/2011 01:50'!
nodeFor: anObject
	^ elementByObjectMap at: anObject ifAbsentPut: (self newNodeFor: anObject)! !

!FLPreviewAnalysis methodsFor: 'opening' stamp: 'MartinDias 12/23/2011 21:02'!
open
	"Example of use:
	(FLAnalyzer newDefault showPreview 
		analysisFor: {1. {{3. {4. {5}}}. 2}})
		open
		"	
	| view nodes |
	view := ROView new.
	nodes := elementByObjectMap values.
	view addAll: nodes.
	view addAll: edges.
	ROTreeLayout on: nodes.
	view @RODraggable @ RODraggableWithVelocity.
	view open.! !

!FLPreviewAnalysis methodsFor: 'opening' stamp: 'MartinDias 12/23/2011 21:25'!
openPathTo: anObject 
	"Example of use:
	(FLAnalyzer newDefault showPreview 
		analysisFor: {1. {{3. {4. {5}}}. 2}})
		openPathTo: 5.
	"
	| view nodes path |
	path := self edgePathTo: anObject.

	nodes := OrderedCollection new.
	nodes add: path first from.
	path do: [:e | nodes add: e to ].

	view := ROView new.
	view addAll: nodes.
	view addAll: path.
	
	ROVerticalLineLayout on: nodes.
	view @RODraggable @ RODraggableWithVelocity.
	view open.! !

!FLPreviewAnalysis methodsFor: 'analyzing' stamp: 'MartinDias 12/23/2011 20:58'!
trace: aReferencedObject
	super trace: aReferencedObject.
	traceForPreview ifTrue: [
		edges add: (self newEdgeTo: aReferencedObject)]! !

!FLAnalyzer methodsFor: '*FuelPreview' stamp: 'MartinDias 12/4/2011 01:27'!
setPreviewAnalysis

	analysisFactory := [:anObject | 
		(FLPreviewAnalysis 
			newWith: self firstInMapperChain 
			root: anObject)
			run;
			yourself ]! !

!FLAnalyzer methodsFor: '*FuelPreview' stamp: 'MartinDias 12/4/2011 01:28'!
showPreview

	self setPreviewAnalysis! !

TestCase subclass: #FLPreviewAnalysisTest
	instanceVariableNames: 'previewAnalysis'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPreview'!

!FLPreviewAnalysisTest methodsFor: 'accessing' stamp: 'MartinDias 12/22/2011 01:51'!
analysis
	^previewAnalysis ! !

!FLPreviewAnalysisTest methodsFor: 'as yet unclassified' stamp: 'MartinDias 12/22/2011 01:49'!
analyze: anObject

	previewAnalysis := FLAnalyzer newDefault showPreview analysisFor: anObject
	! !

!FLPreviewAnalysisTest methodsFor: 'accessing' stamp: 'MartinDias 12/22/2011 01:52'!
edgeTo: anObject
	^ previewAnalysis edges detect: [:e | e to model = anObject ]! !

!FLPreviewAnalysisTest methodsFor: 'as yet unclassified' stamp: 'MartinDias 12/22/2011 01:25'!
previewFor: anObject

	^FLAnalyzer newDefault showPreview analysisFor: anObject
	! !

!FLPreviewAnalysisTest methodsFor: 'tests' stamp: 'MartinDias 12/22/2011 14:08'!
testNestedArrays
	
	| array subsubarray subarray edges |
	subsubarray := {2}.
	subarray := {1. subsubarray}.
	array := {subarray. 0}.
	
	self analyze: array.
	edges := self analysis edges.
	
	(self edgeTo: subarray).
	(self edgeTo: subsubarray).
	(self edgeTo: 2).
! !

!FLPreviewAnalysisTest methodsFor: 'tests' stamp: 'MartinDias 12/22/2011 14:03'!
testPathOfLength1
	
	| array path |
	array := Array with: 1 with: 2 with: 3.
	
	self analyze: array.
	path := self analysis edgePathTo: 2.
	
	self assert: path size = 1.
	self assert: path first = (self edgeTo: 2).! !

!FLPreviewAnalysisTest methodsFor: 'tests' stamp: 'MartinDias 12/22/2011 14:05'!
testPathOfLength3
	
	| array path subsubarray subarray |
	subsubarray := {2}.
	subarray := {1. subsubarray}.
	array := {subarray. 0}.
	
	self analyze: array.
	path := self analysis edgePathTo: 2.
	
	self assert: path size = 3.
	self assert: path first = (self edgeTo: subarray).
	self assert: path second = (self edgeTo: subsubarray).
	self assert: path third = (self edgeTo: 2).
! !
